import pandas as pd
import numpy as np
from sklearn.metrics import f1_score

from tqdm.auto import tqdm

def best_thr_f1(y_true:pd.Series, score:pd.Series, bins:int = 100, out:int=1, r:bool=False):
	scores = list(score.unique()) if type(score) is pd.Series else list(score)
	scores = [max(min(scores) - 0.000001, 0.000001)] + scores
	
	percentiles = np.arange(0, 100, 100/bins)
	thrs = np.percentile(scores, percentiles)
	
	df = pd.concat([y_true, score], axis=1)
	result = []
	
	for thr in tqdm(thrs):
		df['y_pred_thr'] = (df[score.name] > thr) * 1
		
		result.append(
			(thr, f1_score(df[y_true.name], df['y_pred_thr']))
		)
	
	result = pd.DataFrame(result, columns=['thr', 'f1'])
	result = result.sort_values(by='f1', ascending=False)
	
	result = result.head(min(out, len(result)))
	best_thr = result['thr'].loc[result.index[0]]
	
	print()
	print(result)
	
	if r: return round(best_thr, 6)







import pandasflow as pdf
import pandas as pd


y_true = pd.Series([10, 10, 10, 10, 10])
y_pred = pd.Series([5, 5, 7, 5, -5])
y_pred2 = pd.Series([6, 6, 8, 6, 6])

print('default')
prev = pdf.metrics.mean_error(y_true, y_pred2, r=True)
print('\nw\\prev&round')
pdf.metrics.mean_error(y_true, y_pred, previous=prev, round_=3)



import numpy as np



def wape(y_true, y_pred, weights):
	if len(y_true) != len(y_pred) or len(y_pred) != len(weights):
		raise ValueError('Размеры массивов не совпадают!')
	
	errors = np.abs((y_true - y_pred) / y_true)
	wape = np.average(errors, weights=weights)
	
	return wape


import numpy as np

from pandasflow import services
import pandas as pd
from sklearn.metrics import mean_absolute_error, mean_absolute_percentage_error
from sklearn.metrics import mean_squared_error, mean_squared_log_error

def mean_error(y_true: pd.Series, y_pred: pd.Series,
			   previous:  list | tuple | None = None,
			   round_: int | None = None,
			   r: bool = False,
			   ):
	'''print beauty table
	'''
	
	if len(y_true) != len(y_pred):
		raise ValueError('Размеры массивов y_true и y_pred не совпадают!')
	
	mae = mean_absolute_error(y_true, y_pred)
	mape = mean_absolute_percentage_error(y_true, y_pred)
	smape = services.smape(y_true, y_pred)
	rmse = mean_squared_error(y_true, y_pred, squared=False)
	try:
		rmsle = mean_squared_log_error(y_true, y_pred, squared=False)
	except Exception:
		rmsle = -1.111
	
	if round_ is not None:
		mae		= round(mae, round_)
		mape	= round(mape, round_)
		smape	= round(smape, round_)
		rmse	= round(rmse, round_)
		
		try:
			rmsle	= round(rmsle, round_)
		except Exception:
			rmsle	= -1.111
		
	table = pd.DataFrame()
	table.index =	['MAE', 'MAPE', 'SMAPE', 'RMSE', 'RMSLE']
	table['current'] =	[mae, mape, smape, rmse, rmsle]
	
	if previous is not None:
		if len(previous) != len(table.index):
			raise ValueError('list с метриками прошлой итерации имеет неподходящую длину')
		
		table['previous'] = previous
		if round_ is not None: table['previous'] = round(table['previous'], round_)
		
		table['diff'] = (table['current'] - table['previous']) * -1
		if round_ is not None: table['diff'] = round(table['diff'], round_)
		
		table['diff %'] = round(table['diff'] / table['current'] * 100, 2)
	
	print(table)
	
	if r:
		return tuple(table['current'])
